package cn.skcks.study.disruptor;


import com.lmax.disruptor.BusySpinWaitStrategy;
import com.lmax.disruptor.EventFactory;
import com.lmax.disruptor.EventHandler;
import com.lmax.disruptor.RingBuffer;
import com.lmax.disruptor.dsl.Disruptor;
import com.lmax.disruptor.dsl.ProducerType;
import lombok.extern.slf4j.Slf4j;
import org.springframework.scheduling.concurrent.ThreadPoolTaskExecutor;

import java.util.concurrent.ThreadPoolExecutor;

@Slf4j
public class DisruptorTest {
    /**
     * cpu 核心数
     */
    public static final int CPU_NUM = Runtime.getRuntime().availableProcessors();
    /**
     * 最大线程数
     */
    public static final int MAX_POOL_SIZE = CPU_NUM * 2;
    /**
     * 允许线程空闲时间（单位：默认为秒）
     */
    private static final int KEEP_ALIVE_TIME = 20;
    /**
     * 队列长度
     */
    public static final int TASK_NUM = 5;
    /**
     * 线程名称(前缀)
     */
    public static final String THREAD_NAME_PREFIX = "custom-disruptor-executor";

    public static class LongEvent
    {
        private long value;

        public void set(long value)
        {
            this.value = value;
        }

        @Override
        public String toString()
        {
            return "LongEvent{" + "value=" + value + '}';
        }
    }

    public static class LongEventFactory implements EventFactory<LongEvent>
    {
        @Override
        public LongEvent newInstance()
        {
            return new LongEvent();
        }
    }

    public static class LongEventHandler implements EventHandler<LongEvent>
    {
        @Override
        public void onEvent(LongEvent event, long sequence, boolean endOfBatch)
        {
            log.info("Event: {}, sequence: {}, endOfBatch: {}", event, sequence, endOfBatch);
        }
    }

    public static void main(String[] args) throws Exception
    {
        ThreadPoolTaskExecutor executor = new ThreadPoolTaskExecutor();
        executor.setCorePoolSize(CPU_NUM);
        executor.setMaxPoolSize(MAX_POOL_SIZE);
        executor.setQueueCapacity(TASK_NUM);
        executor.setKeepAliveSeconds(KEEP_ALIVE_TIME);
        executor.setThreadNamePrefix(THREAD_NAME_PREFIX);
        // 线程池对拒绝任务的处理策略
        // CallerRunsPolicy：由调用线程（提交任务的线程）处理该任务
        executor.setRejectedExecutionHandler(new ThreadPoolExecutor.CallerRunsPolicy());
        // 初始化
        executor.initialize();

        // ExecutorService executor = Executors.newCachedThreadPool();
        int bufferSize = 64;

        // LongEventFactory factory = new LongEventFactory();
        // Disruptor<LongEvent> disruptor =
        //         new Disruptor<>(factory, bufferSize, DaemonThreadFactory.INSTANCE, ProducerType.SINGLE, new BusySpinWaitStrategy());

        // Disruptor<LongEvent> disruptor =
        //         new Disruptor<>(LongEvent::new, bufferSize, DaemonThreadFactory.INSTANCE, ProducerType.SINGLE, new BusySpinWaitStrategy());

        Disruptor<LongEvent> disruptor =
                new Disruptor<>(LongEvent::new, bufferSize, executor, ProducerType.SINGLE, new BusySpinWaitStrategy());

        // disruptor.handleEventsWith((event, sequence, endOfBatch) ->
        //         System.out.println("Event: " + event));

        disruptor.handleEventsWith(new LongEventHandler(),new LongEventHandler(),new LongEventHandler());
        disruptor.start();

        RingBuffer<LongEvent> ringBuffer = disruptor.getRingBuffer();
        new Thread(()->{
            for (long l = 0; l<256; l++)
            {
                // bb.putLong(0, l);
                // ringBuffer.publishEvent((event, sequence, buffer) -> event.set(buffer.getLong(0)), bb);

                long finalL = l;
                ringBuffer.publishEvent((event, sequence, buffer) -> event.set(finalL));
            }
        }).start();

        // ByteBuffer bb = ByteBuffer.allocate(8);
        for (long l = 0; l<128; l++)
        {
            // bb.putLong(0, l);
            // ringBuffer.publishEvent((event, sequence, buffer) -> event.set(buffer.getLong(0)), bb);

            long finalL = l;
            ringBuffer.publishEvent((event, sequence, buffer) -> event.set(finalL));
            Thread.sleep(1);
        }

        disruptor.shutdown(); //关闭 disruptor  阻塞直至所有事件都得到处理
        executor.shutdown();
    }
}
