package cn.skcks.study.rxjava;

import cn.hutool.core.util.IdUtil;
import cn.hutool.core.util.RandomUtil;
import cn.skcks.docking.gb28181.sip.method.register.request.RegisterRequestBuilder;
import gov.nist.javax.sip.message.SIPRequest;
import io.reactivex.rxjava3.core.Flowable;
import io.reactivex.rxjava3.core.Observable;
import io.reactivex.rxjava3.disposables.Disposable;
import io.reactivex.rxjava3.functions.Consumer;
import io.reactivex.rxjava3.schedulers.Schedulers;
import lombok.SneakyThrows;
import lombok.extern.slf4j.Slf4j;
import org.junit.jupiter.api.Test;
import org.springframework.boot.test.context.SpringBootTest;

import javax.sip.ListeningPoint;
import javax.sip.message.Request;
import java.util.concurrent.TimeUnit;
import java.util.concurrent.atomic.AtomicLong;

@Slf4j
@SpringBootTest(webEnvironment = SpringBootTest.WebEnvironment.NONE)
public class RxJavaStudyStarterTest {
    @Test
    void contextLoads() {
      log.info("Hello, RxJava!");
        Disposable disposable = Flowable.just("Hello world").subscribe(System.out::println);
        disposable.dispose();

        // Observable<Long> longObservable = Observable.create((ObservableOnSubscribe<Long>) emitter -> {
        //     for (long i = 1L; i <= 5L; i++) {
        //         emitter.onNext(i);
        //         Thread.sleep((long) (Math.random() * 100));
        //     }
        //     emitter.onComplete();
        // }).subscribeOn(Schedulers.io())
        //         .observeOn(Schedulers.newThread())
        //         .doOnError(e -> log.error("Error: ", e));

        Observable<Long> intervalObservable = Observable.interval(20, TimeUnit.MILLISECONDS)
                .take(50)
                .subscribeOn(Schedulers.io())
                .observeOn(Schedulers.newThread())
                .doOnError(e -> log.error("Error: ", e));

        Disposable delayDisposable = intervalObservable.subscribe(item -> {
            try {
                log.debug("println: Subscribe Next: {}", item);
                Thread.sleep(100);
            } catch (InterruptedException e) {
                log.error("执行中断 ", e);
            }

        }, e -> {
            log.error("Error: ", e);
        }, () -> log.info("Complete"));

        intervalObservable.subscribe(item -> log.info("[logInfo] Subscribe Next: {}", item));
        new Thread(()->{
            try {
                Thread.sleep(200);
            } catch (InterruptedException ignored) {

            }
            delayDisposable.dispose();
        }).start();
        intervalObservable.blockingSubscribe();

        Observable<Integer> take = Observable.fromArray(1, 2, 3, 4, 5)
                .observeOn(Schedulers.computation()).skip(2).take(1);
        take.subscribe(System.out::println);
    }

    @SneakyThrows
    @Test
    void gb28181SipTest(){
        String localIp = "127.0.0.1";
        int localPort = 5060;
        String localId = "44050100000000000001";

        String serverId = "44050100002000000003";
        String serverIp = "10.10.10.20";
        int serverPort = 5060;

        String domain = "4405010000";

        RegisterRequestBuilder requestBuilder = RegisterRequestBuilder.builder()
                .localId(localId)
                .localIp(localIp)
                .localPort(localPort)
                .targetId(serverId)
                .targetIp(serverIp)
                .targetPort(serverPort)

                .transport(ListeningPoint.UDP)
                .build();

        AtomicLong produce = new AtomicLong(0);
        AtomicLong consume = new AtomicLong(0);

        Flowable<Request> requestObservable = Flowable.intervalRange(0,10,0,100,TimeUnit.MILLISECONDS)
                .onBackpressureDrop()
                .map(i -> {
                    String callId = IdUtil.fastSimpleUUID();
                    produce.getAndAdd(1);
                    return requestBuilder.createNoAuthorizationRequest(callId, 3600);
                })
                .cache()
                .share()
                .distinct(request -> ((SIPRequest) request).getCallId().getCallId())
                .observeOn(Schedulers.computation())
                .subscribeOn(Schedulers.io());

        Flowable<Request> requestFlowable = requestObservable.publish().refCount();

        Consumer<Request> serverConsumer = request -> {
            SIPRequest sipRequest = (SIPRequest) request;
            String method = request.getMethod();
            log.info("收到SIP请求: {}", method);
            Thread.sleep(RandomUtil.randomLong(10,100));
            log.info("处理SIP请求: {}", sipRequest.getCallId().getCallId());
            consume.getAndAdd(1);
        };

        requestFlowable.subscribe(serverConsumer);
        requestFlowable.subscribe(serverConsumer);
        // requestObservable.subscribe(serverConsumer);
        // requestObservable.blockingSubscribe();

        Thread.sleep(TimeUnit.SECONDS.toMillis(30));
        log.info("生产数量: {}, 消费数量: {}", produce.get(), consume.get());
    }
}
